/*
 * Copyright 2015-2018 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opentest4j;

/**
 * Specialization of {@link IncompleteExecutionException} used to indicate
 * that a test was skipped <em>prior</em> to execution (e.g., <em>disabled</em>
 * or <em>ignored</em>).
 *
 * @author Sam Brannen
 * @author Johannes Link
 * @since 1.0
 * @see TestAbortedException
 */
public class TestSkippedException extends IncompleteExecutionException {

	private static final long serialVersionUID = 1L;

	public TestSkippedException() {
	}

	public TestSkippedException(String message) {
		super(message);
	}

	public TestSkippedException(String message, Throwable cause) {
		super(message, cause);
	}

}
